#include <stdint.h>
#include <string.h>
#include "nordic_common.h"
#include "nrf.h"
#include "app_error.h"
#include "nrf51_bitfields.h"

#include "softdevice_handler.h"

#include "pstorage.h"


#define DEAD_BEEF                            0xDEADBEEF                                /**< Value used as error code on stack dump, can be used to identify stack location on stack unwind. */



/*****************************************************************************
* Main Function
*****************************************************************************/
void app_error_handler(uint32_t error_code, uint32_t line_num, const uint8_t * p_file_name)
{
    // This call can be used for debug purposes during development of an application.
    // @note CAUTION: Activating this code will write the stack to flash on an error.
    //                This function should NOT be used in a final product.
    //                It is intended STRICTLY for development/debugging purposes.
    //                The flash write will happen EVEN if the radio is active, thus interrupting
    //                any communication.
    //                Use with care. Un-comment the line below to use.
    // ble_debug_assert_handler(error_code, line_num, p_file_name);

    // On assert, the system can only recover with a reset.
    NVIC_SystemReset();
}

void assert_nrf_callback(uint16_t line_num, const uint8_t * p_file_name)
{
    app_error_handler(DEAD_BEEF, line_num, p_file_name);
}


static void ble_evt_dispatch(ble_evt_t * p_ble_evt)
{
}

static void sys_evt_dispatch(uint32_t sys_evt)
{
    pstorage_sys_event_handler(sys_evt);
}

static void storage_cb_handler(pstorage_handle_t  * handle,
                               uint8_t              op_code,
                               uint32_t             result,
                               uint8_t            * p_data,
                               uint32_t             data_len)
{
  uint32_t res = result;
  if(res != NRF_SUCCESS) {
    ;//do nothing ?
  } else{

  }
}


static void power_manage(void)
{
    uint32_t err_code = sd_app_evt_wait();
    APP_ERROR_CHECK(err_code);
}



/**@brief Application main function.
 */
int main(void)
{
  uint32_t err_code;
  
  // Initialize the SoftDevice handler module.
  SOFTDEVICE_HANDLER_INIT(NRF_CLOCK_LFCLKSRC_RC_250_PPM_4000MS_CALIBRATION/*NRF_CLOCK_LFCLKSRC_XTAL_20_PPM*/, false);


  // Register with the SoftDevice handler module for BLE events.
  err_code = softdevice_ble_evt_handler_set(ble_evt_dispatch);
  APP_ERROR_CHECK(err_code);
  
  // Register with the SoftDevice handler module for BLE events.
  err_code = softdevice_sys_evt_handler_set(sys_evt_dispatch);
  APP_ERROR_CHECK(err_code);
  
  
  //init persistent storage
  pstorage_module_param_t param;
  pstorage_handle_t       base_handle;
  pstorage_handle_t       first_handle;

  err_code = pstorage_init();
  APP_ERROR_CHECK(err_code);

    
  param.block_size  = 20;
  param.block_count = 1;
  param.cb          = storage_cb_handler;


  
  err_code = pstorage_register(&param, &base_handle);
  APP_ERROR_CHECK(err_code);

  err_code = pstorage_block_identifier_get(&base_handle, 0, &first_handle);
  APP_ERROR_CHECK(err_code);

  uint8_t data[12] = "Something";
  uint8_t loadedData[12];
  err_code = pstorage_load(loadedData, &first_handle, 12, 0);
  APP_ERROR_CHECK(err_code);
  if(memcmp(loadedData, data, 9) != 0) {
    //It always enter here
    err_code = pstorage_store(&first_handle, data, 10, 0);
    APP_ERROR_CHECK(err_code);
  }


  
  // Enter main loop.
  for (;;) {
    power_manage();
  }


}
